/*
 * Copyright (c) 2005
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Author(s): Philipp Scholl <scholl@teco.edu>
 */

package edu.teco.particles.pComponent.io;

import edu.teco.particles.libParticle.ParticlePacket;

import java.awt.event.*;
import java.util.*;

/**
 * Template that always matches.
 * 
 * stores the packet the template matched on.
 */
public class DefaultTemplate implements ParticleTemplate
{
  private ParticlePacket _matchedPacket;

  public boolean match(ParticlePacket packet)
  {
    _matchedPacket = packet;
    _notifyActionListeners();
    return true;
  }

  public Object parse(ParticlePacket packet)
  {
    return packet;
  }

  public Object parse()
  {
    return parse(_matchedPacket);
  }

  public ParticlePacket packet(Object obj, ParticlePacket pck)
  {
    return pck;
  }

  /**
   * @name ActionListener. @{
   */
  private LinkedList<ActionListener> _listeners = new LinkedList<ActionListener>();

  public synchronized void addActionListener(ActionListener action)
  {
    if (!_listeners.contains(action)) _listeners.add(action);
  }

  public synchronized void removeActionListener(ActionListener action)
  {
    _listeners.remove(action);
  }

  public List<ActionListener> getActionListeners()
  {
    return _listeners;
  }

  protected synchronized void _notifyActionListeners()
  {
    ActionEvent event = new ActionEvent(this, 0, MATCH_CMD);
    for (ActionListener listener : _listeners)
    {
      listener.actionPerformed(event);
    }
  }

  /** @} */

  protected Object _userObject = null;

  public void setUserObject(Object o)
  {
    _userObject = o;
  }

  public Object getUserObject()
  {
    return _userObject;
  }

  public Date getTimeStamp()
  {
    return new Date(System.currentTimeMillis());
  }

  public void addChild(ParticleTemplate tupleTemplate)
  {
    // TODO Auto-generated method stub
  }

  public LinkedList<ParticleTemplate> getChilds()
  {
    // TODO Auto-generated method stub
    return null;
  }

}
