/*
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package edu.teco.particles.pComponent.io;

import java.awt.event.ActionListener;
import java.util.*;

import edu.teco.particles.libParticle.ParticlePacket;

/**
 * defines methods for templates which can be used to identify data in a
 * particleNetwork.
 * 
 * ActionListener support to be notified if a Template matched.
 */
public interface ParticleTemplate
{
  public final String MATCH_CMD = "Template Matched.";

  /**
   * matches this template against a tuple.
   */
  public abstract boolean match(ParticlePacket packet);

  /**
   * parses a given tuple.
   * 
   * deserialization from a ParticleTuple.
   */
  public abstract Object parse(ParticlePacket packet)
      throws IllegalArgumentException;

  /**
   * parses the packet which last matched this template.
   */
  public Object parse() throws IllegalArgumentException;

  /**
   * creates a new Tuple given the data in object.
   * 
   * serialization to a ParticleTuple.
   */
  public abstract ParticlePacket packet(Object obj, ParticlePacket pck)
      throws IllegalArgumentException;

  /**
   * remove an ActionListener.
   */
  public void removeActionListener(ActionListener action);

  /**
   * add an ActionListeners, which gets called if this template matched.
   */
  public void addActionListener(ActionListener action);

  /**
   * get the curent ActionListeners.
   */
  public List<ActionListener> getActionListeners();

  /**
   * associate a user specified object with this template.
   */
  public void setUserObject(Object o);

  /**
   * get the associated object.
   */
  public Object getUserObject();

  public void addChild(ParticleTemplate tupleTemplate);

  public LinkedList<ParticleTemplate> getChilds();
}
