package edu.teco.particles.pComponent.test;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Timer;

import edu.teco.particles.libParticle.ParticleSocket;
import edu.teco.particles.pComponent.Multiplexer;
import edu.teco.particles.pComponent.io.ParticleTemplate;
import junit.framework.TestCase;

/**
 * General Purpose Testcase in particle enhanced Environment.
 * 
 * Features: - assertMatch(ParticleTemplate) method
 * 
 * @author Philipp Scholl <scholl@teco.edu>
 * 
 */
public class ParticleTestCase extends TestCase
{
  protected Multiplexer    _multiplexer;

  private ActionListener   _actionListener;

  protected ParticleSocket _sendSocket;

  static
  {
    System.loadLibrary("particleJava");
  }

  public void setUp() throws Exception
  {
    super.setUp();

    _multiplexer = new Multiplexer(5555);
    _sendSocket = new ParticleSocket(5555);

    _actionListener = new ActionListener() {
      public synchronized void actionPerformed(ActionEvent e)
      {
        this.notify();
      }
    };
  }

  public void tearDown() throws Exception
  {
    super.tearDown();
    _multiplexer.clear();
  }

  protected void assertMatch(ParticleTemplate template)
  {
    template.addActionListener(_actionListener);
    _multiplexer.addTemplate(template);
  }

  protected void waitForMatch()
  {
    synchronized (_actionListener)
    {
      try
      {
        _actionListener.wait(1000);
      }
      catch (InterruptedException e)
      {
        e.printStackTrace();
      }
    }
  }
}
