/*
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * @author Philipp Scholl <scholl@teco.edu>
 */

package edu.teco.particles.pComponent.util;

import edu.teco.particles.libParticle.ParticleTuple;

/**
 * Parser for two byte, little endian, signed/unsigned to Java Long.
 */
public class LongParser extends ShortParser
{

  public LongParser()
  {
    super();
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#parse(edu.teco.particles.libParticle.ParticleTuple)
   */
  public Object parse(ParticleTuple pt) throws IllegalArgumentException
  {
    if (pt.length() < getOffset() + 2)
    {
      throw new IllegalArgumentException("tuple is too short.");
    }

    long value;
    short arr[] = pt.toArray();
    int i = getOffset();

    if (isSigned())
    {
      value = arr[i] << 8 & 0xFF00;
      value |= arr[i + 1] & 0x00FF;
      value = (short) value;
    }
    else
    {
      value = arr[i] << 8 & 0xFF00;
      value |= arr[i + 1] & 0x00FF;
    }

    return new Long(value);
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#serialize(java.lang.Object)
   */
  public short[] serialize(Object obj) throws IllegalArgumentException
  {
    short[] sarr = super.serialize(obj);

    if (obj instanceof Integer)
    {
      Integer num = (Integer) obj;
      sarr = new short[] { 0, 0 };

      sarr[0] |= num.shortValue() >> 8;
      sarr[1] |= num.byteValue();
    }

    return sarr;
  }

  /*
   * (non-Javadoc)
   * 
   * @see edu.teco.particles.pComponent.util.TupleParser#getDataType()
   */
  public String getDataType()
  {
    return "Long (16bit)";
  }

}
