/*
 * Created on 18.04.2005
 *
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package edu.teco.particles.pComponent.util;

import java.awt.Color;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Random;

import edu.teco.particles.libParticle.ParticleSrcId;
import edu.teco.particles.libParticle.ParticleTuple;

/**
 * 
 */
public class ParticleContext
{
  protected ParticleTuple    _tuple;

  protected ParticleSrcId    _id;

  protected Color            _color;

  static public final String COLOR = "color";

  public ParticleContext()
  {
    Random rand = new Random();
    _color = new Color(rand.nextInt(255), rand.nextInt(255), rand.nextInt(255));
  }

  public ParticleContext(ParticleSrcId id, ParticleTuple tuple)
  {
    _tuple = tuple;
    _id = id;
    Random rand = new Random();
    _color = new Color(rand.nextInt(255), rand.nextInt(255), rand.nextInt(255));
  }

  public void setColor(Color col)
  {
    _supp.firePropertyChange(COLOR, _color, col);
    _color = col;
  }

  public Color getColor()
  {
    return _color;
  }

  public void setTuple(ParticleTuple tup)
  {
    _tuple = tup;
  }

  public ParticleTuple getTuple()
  {
    return _tuple;
  }

  public void setSrcId(ParticleSrcId id)
  {
    _id = id;
  }

  public ParticleSrcId getSrcId()
  {
    return _id;
  }

  public boolean equals(Object other)
  {
    if (!(other instanceof ParticleContext)) return false;

    ParticleContext o = (ParticleContext) other;

    if (!(o.getSrcId().equals(_id))) return false;

    if (!(o.getTuple().equals(_tuple))) return false;

    return true;
  }

  public int hashCode()
  {
    return _id.hashCode() + _tuple.getAclType().hashCode();
  }

  public String toString()
  {
    String str = "";
    str += _tuple.getAclType();
    str += " " + _id.toString();
    return str;
  }

  /*
   * ===========================================================================
   * Property Observer part of this class delegate to PropertyChangeSupport.
   * ========================================================================
   */
  protected PropertyChangeSupport _supp = new PropertyChangeSupport(this);

  public void addPropertyChangeListener(PropertyChangeListener list)
  {
    _supp.addPropertyChangeListener(list);
  }

  public void removePropertyChangeListener(PropertyChangeListener list)
  {
    _supp.removePropertyChangeListener(list);
  }
}
