/*
 * Copyright (c) 2004
 * Telecooperation Office (TecO), Universitaet Karlsruhe (TH), Germany.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the Universitaet Karlsruhe (TH) nor the names
 *    of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * @author Philipp Scholl <scholl@teco.edu>
 */

package edu.teco.particles.pComponent.util;

import java.util.Collection;
import java.util.LinkedList;

/**
 * A RingBuffer Implementation.
 * 
 * modified LinkedList. Set a fixed capacity on Instanation. If the capacity is
 * hit by the add function the element at the start of the list removes. So
 * implementing a LIFO strategy.
 */
public class RingBuffer<E> extends LinkedList<E>
{
  /** the capacity of this RingBuffer */
  protected int _limit;

  /**
   * creates a new RingBuffer.
   * 
   * @param limit
   *          the maximum size of the Ringbuffer
   */
  public RingBuffer(int limit)
  {
    _limit = limit;
  }

  /** adds a element to the collection */
  public boolean add(E element)
  {
    super.add(element);

    if (size() > _limit)
    {
      removeFirst();
      assert (size() == _limit) : "size: " + size();
      return true;
    }
    else
    {
      return false;
    }
  }

  public boolean addAll(Collection<? extends E> arg0)
  {
    boolean result = false;
    super.addAll(arg0);

    while (size() > _limit)
    {
      removeFirst();
      result = true;
    }

    return result;
  }
}
